//==============================================================================
// File:    notes.cpp
// Author:  
// Date:    27-Apr-2021
// Purpose: A note taking app.
//==============================================================================
#include <iostream>
#include <fstream>
#include <vector>
using namespace std;

struct Note {
    string label, text;
};

/**
 * Reads in a list of notes from the given file and returns it. Each note in
 * the file should have two lines:
 * 
 * <label>
 * <description>
 * 
 * @param filename The name of the file to read the notes in from.
 * @return A list of Notes read in from the file.
 */
vector<Note> LoadNotes(string filename){
    vector<Note> notes;

    // TODO 1: Implement this function.
    // Steps:
    //
    // declare the input file stream
    // open the input file stream
    // if the file didn't open successfully:
        // output an error message and exit the program (like this: `exit(1);`)
    // while the input stream is good:
        // create a new Note
        // read in the next line of the input stream into the `label` field
        // read in the next line of the input stream into the `text` field
        // if the stream didn't fail:
            // append the Note to the end of notes
    // close the file stream

    return notes;
}

/**
 * Saves the given list of notes to a file. Each note is written across two
 * lines of the output file:
 * 
 * <label>
 * <description>
 * 
 * @param notes The list of Notes to save.
 * @param filename The name of the file to save the notes to.
 */
void SaveNotes(vector<Note> notes, string filename){
    // TODO 2: Implement this function.
    // Steps:
    // 
    // declare the output file stream
    // open the output file stream
    // for each note in notes:
        // write the `label` field to its own line in the output stream
        // write the `text` field to its own line in the output stream
    // close the file
}

/**
 * Reads a note from the user.
 * 
 * @return A Note populated with user input.
 */
Note ReadNoteFromUser(){
    Note note;
    cout << "Enter the following information about your note" << endl
         << "Label: ";
    getline(cin, note.label);
    cout << "Note text: ";
    getline(cin, note.text);

    return note;
}

/**
 * Removes a the Note at the given index from the list of Notes.
 * 
 * @param notes A list of Notes (gets updated).
 * @param indexToRemove The index of the Note to remove.
 */
void RemoveNote(vector<Note> &notes, int indexToRemove){
    for(int i = indexToRemove+1; i < notes.size(); i++){
        notes[i-1] = notes[i];
    }
    notes.resize(notes.size()-1);
}

/**
 * Prints each Note to the console in a nice format along with its index.
 * 
 * @param notes A list of Notes to display.
 */
void DisplayNotes(vector<Note> notes){
    cout << endl << "NOTES" << endl << "=========" << endl;
    for(int i = 0; i < notes.size(); i++){
        cout << i << ". " << "[" << notes[i].label << "]" << endl 
             << notes[i].text << endl << endl;
    }
    cout << endl;
}

/**
 * Provides an interface for an interactive note tracker.
 * 
 * @return The exit status (0 is good).
 */
int main(){
    const string DATABASE = "notes.db";
    char userOption;
    string filename;
    vector<Note> notes;
    int index;

    cout << string(80,'=') << endl
         << string(35, ' ') << "N O T E S" << endl
         << string(80, '-') << endl;


    // Loop until the user selects to quit.
    do {
        // Display the menu and get the user's selection.
        cout << "Select an option:" << endl 
             << "  [l]oad a set of notes from a file (overwrites current notes)" << endl 
             << "  [s]ave the current set of notes to a file" << endl 
             << "  [a]dd a new note" << endl 
             << "  [r]emove a note from the list" << endl 
             << "  [q]uit" << endl
             << "> ";
        cin >> userOption;
        cin.ignore();

        switch(userOption){
            // Load a file.
            case 'l':
                cout << "Filename to load: ";
                getline(cin, filename);
                notes = LoadNotes(filename);
                break;

            // Save notes to a file.
            case 's':
                cout << "Filename to save notes to: ";
                getline(cin, filename);
                SaveNotes(notes, filename);
                break;

            // Add a new note.
            case 'a':
                notes.push_back(ReadNoteFromUser());
                break;

            // Remove a note.
            case 'r':
                cout << "Index to remove: ";
                cin >> index;
                cin.ignore();
                RemoveNote(notes, index);
                break;

            case 'q':
                cout << "Are you sure you want to quit? [y/n]: ";
                cin >> userOption;
                cin.ignore();
                if(userOption == 'y'){
                    userOption = 'q';
                }
        }

        // List all of the notes as long as the user hasn't selected to quit.
        if(userOption != 'q'){
            DisplayNotes(notes);
        }

    } while(userOption != 'q');

    cout << "Bye!" << endl;

    return 0;
}
